/* 
 * Blinky - a GTK1 network usage monitor
 *
 * Copyright (C) 2006, Lior Tubi <lior2b@gmail.com>
 *
 * This Blinky release is based on a work by Benjamin Miller <bmiller@users.sf.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include "blinky.h"

#define BLINKY_VERSION "0.8"

/* global variables */
GtkWidget *mainbox;
GSList *interface_list = NULL; /* list of interface structs */
GtkTooltips *tips;
blinky_opts options;
image img_idle, img_tx, img_rx, img_txrx;

GtkWidget *show_main_window(int, char **);
void hide_close(GtkButton *w, gpointer window);
void hide_quit(GtkWidget *w, gpointer dummy);
void check_toggled(GtkCheckButton *, gpointer);


/****************************************************************************
 ****************************************************************************/

int main(int argc, char **argv) {
    static guint32 timer;
    GtkWidget *mainwin;
	int i;

    gtk_init(&argc, &argv);

	options.bgarg = 0;
	for(i = 1; i < argc; ++i) {
		if(!strcmp(argv[i], "-v") || !strcmp(argv[i], "--version")) {
			printf("Blinky v%s by Lior Tubi <lior2b@gmail.com>\n", BLINKY_VERSION);
			return 0;
		} else if(!strcmp(argv[i], "-bg") && i<argc-1) {
			if(gdk_color_parse(argv[i+1], &(options.bgcolor)))
				options.bgarg = 1;
		}
	}

    mainbox = gtk_hbox_new(FALSE, 0);
	tips = gtk_tooltips_new();

    mainwin = show_main_window(argc, argv);

    get_setup();
    update_all(NULL);
    timer = gtk_timeout_add(options.interval, update_all, NULL);

    gtk_main();

    return 0;
}

/****************************************************************************
 ****************************************************************************/

GtkWidget *show_main_window(int argc, char **argv) {
    GtkWidget *window;
	GtkStyle *style;

    window = gtk_window_new(GTK_WINDOW_TOPLEVEL); /* create main window */
    gtk_signal_connect(GTK_OBJECT(window), "destroy", GTK_SIGNAL_FUNC(quit), NULL);
    gtk_signal_connect(GTK_OBJECT(window), "destroy_event", GTK_SIGNAL_FUNC(quit), NULL);
    gtk_window_set_policy(GTK_WINDOW(window), FALSE, FALSE, TRUE);
    gtk_window_set_title(GTK_WINDOW(window), "Blinky");
    
	if(options.bgarg) {
		style = gtk_style_copy(gtk_widget_get_style(window));
		style->bg[GTK_STATE_NORMAL] = options.bgcolor;
		gtk_widget_set_style (window, style);
		gtk_widget_set_style (mainbox, style);
	}

    gtk_widget_realize(window);
	
	gtk_container_set_border_width(GTK_CONTAINER(window), 1);
    gtk_container_add(GTK_CONTAINER(window), mainbox);

    gtk_widget_show_all(window);

    return window;
}

/****************************************************************************
 ****************************************************************************/

void show_hide_window(void) {
	GSList *l;
	GtkWidget *window, *widget, *vbox, *frame, *box;
	char note[256], temp[128], tx_str[16], rx_str[16];;

	if(options.options_window != NULL ) {
		/* gtk1.2 doesn't have gtk_window_present() */
		gtk_widget_hide(options.options_window);
		gtk_widget_show(options.options_window);
		return;
	}

	g_snprintf(note, sizeof(note), "Configuration file will be saved as:\n%s/.blinky", getenv("HOME"));

	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	options.options_window = window;
    gtk_signal_connect(GTK_OBJECT(window), "destroy", GTK_SIGNAL_FUNC(hide_quit), NULL);
    gtk_signal_connect(GTK_OBJECT(window), "destroy_event", GTK_SIGNAL_FUNC(hide_quit), NULL);
	gtk_window_set_title(GTK_WINDOW(window), "Options - Blinky");
    gtk_container_set_border_width(GTK_CONTAINER(window), 8);
	gtk_window_set_policy(GTK_WINDOW(window), FALSE, FALSE, TRUE);
	gtk_widget_realize(window);
	
	vbox = gtk_vbox_new(FALSE, 5);
	gtk_container_add(GTK_CONTAINER(window), vbox);

	frame = gtk_frame_new("Hide / Show Connections");
	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 3);

	box = gtk_vbox_new(FALSE, 5);
	gtk_container_set_border_width(GTK_CONTAINER(box), 10);
	gtk_container_add(GTK_CONTAINER(frame), box);

	widget = gtk_label_new("Check the connections you wish to hide:");
    gtk_box_pack_start(GTK_BOX(box), widget, FALSE, FALSE, 3);

	l = interface_list;
	while(l != NULL) {
		if(strcmp((char *)(((interface *)l->data)->name), "lo")) {
			format_bytes(((interface *)(l->data))->rx_cur, rx_str, sizeof(rx_str));
			format_bytes(((interface *)(l->data))->tx_cur, tx_str, sizeof(tx_str));
			g_snprintf(temp, sizeof(temp), "%s [ Received: %s | Sent: %s ]", (char *)(((interface *)l->data)->name), rx_str, tx_str);
			widget = gtk_check_button_new_with_label(temp);
			gtk_signal_connect(GTK_OBJECT(widget), "toggled", GTK_SIGNAL_FUNC(check_toggled), (interface *)l->data);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), (gboolean)(((interface *)l->data)->hidden));
		    gtk_box_pack_start(GTK_BOX(box), widget, FALSE, FALSE, 0);
		}
		l = l->next;
	}

	frame = gtk_frame_new("Note");
	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 3);

	widget = gtk_label_new(note);
	gtk_container_add(GTK_CONTAINER(frame), widget);

	frame = gtk_frame_new("About");
	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 3);

	box = gtk_vbox_new(FALSE, 5);
	gtk_container_set_border_width(GTK_CONTAINER(box), 10);
	gtk_container_add(GTK_CONTAINER(frame), box);

	g_snprintf(temp, sizeof(temp), "Blinky v%s\nGTK1 network usage monitor\nLior Tubi <lior2b@gmail.com>", BLINKY_VERSION);
	widget = gtk_label_new(temp);
	gtk_box_pack_start(GTK_BOX(box), widget, FALSE, FALSE, 3);

    widget = gtk_button_new_with_label("Close");
	gtk_signal_connect(GTK_OBJECT(widget), "clicked", GTK_SIGNAL_FUNC(hide_close), window);
	gtk_box_pack_start(GTK_BOX(vbox), widget, FALSE, FALSE, 0);

	gtk_widget_show_all(window);
}

/****************************************************************************
 ****************************************************************************/

void hide_close(GtkButton *w, gpointer window) {
	options.options_window = NULL;
	gtk_widget_destroy(GTK_WIDGET(window));
}

/****************************************************************************
 ****************************************************************************/

void hide_quit(GtkWidget *w, gpointer dummy) {
	options.options_window = NULL;
}

/****************************************************************************
 ****************************************************************************/

void check_toggled(GtkCheckButton *w, gpointer pointer) {
	interface *iface;

	iface = pointer;
	if(iface->hidden != gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w))) {
		iface->hidden = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
		write_config_file();
	}
}

/****************************************************************************
 ****************************************************************************/

void quit(GtkWidget *w, gpointer dummy) {
    gtk_main_quit();
}
