/* 
 * Blinky - a GTK1 network usage monitor
 *
 * Copyright (C) 2006, Lior Tubi <lior2b@gmail.com>
 *
 * This Blinky release is based on a work by Benjamin Miller <bmiller@users.sf.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */


#ifndef _BLINKY_H
#define _BLINKY_H

#include <gtk/gtk.h>


/* global types */

typedef enum {
	IFACE_IDLE = 0,
	IFACE_TX = 1,
	IFACE_RX = 2,
	IFACE_TXRX = 3
} iface_status;

typedef struct {
    char name[16];
    unsigned long tx_cur, rx_cur;
    gboolean txing, rxing, found, hidden;
    GtkWidget *hbox, *eventbox, *image;
	iface_status old_status;
} interface;

typedef struct {
    int interval;
	GtkWidget *options_window;
	GdkColor bgcolor;
	int bgarg;
} blinky_opts;

typedef struct {
	GdkPixmap *pixmap;    
	GdkBitmap *bitmap;
} image;


/* global variables - blinky.c */
extern GtkWidget *mainbox;
extern GSList *interface_list;
extern GtkTooltips *tips;
extern blinky_opts options;
extern image img_idle, img_tx, img_rx, img_txrx;


/* functions in blinky.c */
void show_hide_window(void);
void quit(GtkWidget *, gpointer);
/* functions in util.c */
gboolean update_all(gpointer);
void get_setup(void);
void write_config_file(void);
void format_bytes(unsigned long long, char *, int);
/* functions in interface.c */
interface *interface_new(const char[]);
void interface_draw(GtkWidget *, GdkEvent *, interface *);
/* functions in usage-proc.c */
void get_usage_all(void);


#endif /* _BLINKY_H */
