/* 
 * Blinky - a GTK1 network usage monitor
 *
 * Copyright (C) 2006, Lior Tubi <lior2b@gmail.com>
 *
 * This Blinky release is based on a work by Benjamin Miller <bmiller@users.sf.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <string.h>
#include <gtk/gtk.h>
#include "blinky.h"

void interface_handle_click(GtkWidget *, GdkEventButton *, interface *);

/****************************************************************************
 ****************************************************************************/

interface *interface_new(const char name[]) {
    interface *iface = g_malloc(sizeof(interface));
    char tip[36];

    if(iface == NULL)
		return NULL;

    iface->tx_cur = iface->rx_cur = 0;
    iface->txing = iface->rxing = iface->found = FALSE;
    strcpy(iface->name, name);
	iface->image = NULL;
	iface->hidden = FALSE;
	iface->old_status = IFACE_IDLE;
	g_snprintf(tip, sizeof(tip), "Network Connection: %s", iface->name);

    iface->hbox = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_end(GTK_BOX(mainbox), iface->hbox, FALSE, FALSE, 3);

    iface->eventbox = gtk_event_box_new();
    gtk_box_pack_start(GTK_BOX(iface->hbox), iface->eventbox, FALSE, FALSE, 0);
    gtk_widget_set_usize(iface->eventbox, 22, 22);
	gtk_widget_set_style (iface->eventbox, gtk_widget_get_style(mainbox));

    gtk_signal_connect_after(GTK_OBJECT(iface->eventbox), "expose_event", GTK_SIGNAL_FUNC(interface_draw), iface);
    gtk_signal_connect(GTK_OBJECT(iface->eventbox), "button_press_event", GTK_SIGNAL_FUNC(interface_handle_click), iface);
    gtk_tooltips_set_tip(tips, iface->eventbox, tip, "");

    return iface;
}

/****************************************************************************
 ****************************************************************************/

void interface_draw(GtkWidget *widget, GdkEvent *event, interface *iface) {
	GtkWidget *image;
	
	if(iface->found == FALSE || iface->hidden == TRUE) {
		gtk_widget_hide(iface->hbox);
		return;
	} else {
		gtk_widget_show_all(iface->hbox);
	}

	image = iface->image;

    /* display computers */
	if(iface->rxing) {
		if(iface->txing) {
			if(iface->image == NULL) {
				image = gtk_pixmap_new(img_txrx.pixmap, img_txrx.bitmap);
				gtk_container_add(GTK_CONTAINER(widget), image);
			    gtk_widget_show(image);
			} else if(iface->old_status != IFACE_TXRX) {
				gtk_container_remove(GTK_CONTAINER(widget), iface->image);
				image = gtk_pixmap_new(img_txrx.pixmap, img_txrx.bitmap);
				gtk_container_add(GTK_CONTAINER(widget), image);
			    gtk_widget_show(image);
			}
			iface->old_status = IFACE_TXRX;
		}else {
			if(iface->image == NULL) {
				image = gtk_pixmap_new(img_rx.pixmap, img_rx.bitmap);
				gtk_container_add(GTK_CONTAINER(widget), image);
			    gtk_widget_show(image);
			} else if(iface->old_status != IFACE_RX) {
				gtk_container_remove(GTK_CONTAINER(widget), iface->image);
				image = gtk_pixmap_new(img_rx.pixmap, img_rx.bitmap);
				gtk_container_add(GTK_CONTAINER(widget), image);
			    gtk_widget_show(image);
			}
			iface->old_status = IFACE_RX;
		}
	} else {
		if(iface->txing) {
			if(iface->image == NULL) {
				image = gtk_pixmap_new(img_tx.pixmap, img_tx.bitmap);
				gtk_container_add(GTK_CONTAINER(widget), image);
			    gtk_widget_show(image);
			} else if(iface->old_status != IFACE_TX) {
				gtk_container_remove(GTK_CONTAINER(widget), iface->image);
				image = gtk_pixmap_new(img_tx.pixmap, img_tx.bitmap);
				gtk_container_add(GTK_CONTAINER(widget), image);
			    gtk_widget_show(image);
			}
			iface->old_status = IFACE_TX;
		} else {
			if(iface->image == NULL) {
				image = gtk_pixmap_new(img_idle.pixmap, img_idle.bitmap);
				gtk_container_add(GTK_CONTAINER(widget), image);
			    gtk_widget_show(image);
			} else if(iface->old_status != IFACE_IDLE) {
				gtk_container_remove(GTK_CONTAINER(widget), iface->image);
				image = gtk_pixmap_new(img_idle.pixmap, img_idle.bitmap);
				gtk_container_add(GTK_CONTAINER(widget), image);
			    gtk_widget_show(image);
			}
			iface->old_status = IFACE_IDLE;
		}
	}
	iface->image = image;
}

/****************************************************************************
 ****************************************************************************/

void interface_handle_click(GtkWidget *widget, GdkEventButton *event, interface *iface) {
    static GtkWidget *menu = NULL;
    GtkWidget *item;
    char buf[32], tx_str[16], rx_str[16];

    if(event->type == GDK_BUTTON_PRESS && event->button == 3) {
		if(menu)
		    gtk_widget_destroy(menu);

		menu = gtk_menu_new();

		format_bytes(iface->rx_cur, rx_str, sizeof(rx_str));
		format_bytes(iface->tx_cur, tx_str, sizeof(tx_str));
			      
		g_snprintf(buf, 32, "Connection: %s",	iface->name);
		item = gtk_menu_item_new_with_label(buf);
		gtk_menu_append(GTK_MENU(menu), item);
		gtk_widget_show(item);

		g_snprintf(buf, 32, "Received: %s", rx_str);
		item = gtk_menu_item_new_with_label(buf);
		gtk_menu_append(GTK_MENU(menu), item);
		gtk_widget_show(item);

		g_snprintf(buf, 32, "Sent: %s", tx_str);
		item = gtk_menu_item_new_with_label(buf);
		gtk_menu_append(GTK_MENU(menu), item);
		gtk_widget_show(item);

		item = gtk_menu_item_new_with_label("Options...");
		gtk_menu_append(GTK_MENU(menu), item);
		gtk_signal_connect(GTK_OBJECT(item), "activate",
				    GTK_SIGNAL_FUNC(show_hide_window), (void *)3);
		gtk_widget_show(item);

		item = gtk_menu_item_new_with_label("Quit Blinky");
		gtk_menu_append(GTK_MENU(menu), item);
		gtk_signal_connect(GTK_OBJECT(item), "activate",
				    GTK_SIGNAL_FUNC(quit), (void *)3);
		gtk_widget_show(item);

		gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, 
			       event->button, event->time);
    }
}
