/* 
 * Blinky - a GTK1 network usage monitor
 *
 * Copyright (C) 2006, Lior Tubi <lior2b@gmail.com>
 *
 * This Blinky release is based on a work by Benjamin Miller <bmiller@users.sf.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */


/* usage-proc.c - get device usage using the Linux /proc/net/dev interface */

#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <stdlib.h>
#include "blinky.h"


static char *parse_line(char *);

/****************************************************************************
 ****************************************************************************/

/* uses /proc/net/dev to get tx and rx counts for all the interfaces */
void get_usage_all(void) {
	int fd;
	static char buf[2048];
	char *pos;

	if((fd = open("/proc/net/dev", O_RDONLY)) == -1) {
		perror("opening /proc/net/dev");
		exit(1);
	}

	memset(buf, 0, sizeof(buf));

	if(read(fd, buf, sizeof(buf)) == -1) {
		perror("reading /proc/net/dev");
		exit(1);
	}

	close(fd);

	pos = buf;

	while(*(pos++) != '\n') ;
	while(*(pos++) != '\n') ;

	/* parse each line */
	while(pos != NULL)
	pos = parse_line(pos);


	return;
}

/****************************************************************************
 ****************************************************************************/

/* parses one line out of /proc/net/dev and
	fills in the appropriate interface if it matches any */
static char *parse_line(char *pos) {
	unsigned long tx, rx;
	char name[16];
	interface *iface = NULL;
	GSList *l;

	/* get name */
	while(*pos == ' ') pos++;
	sscanf(pos, "%s", name);
	name[strcspn(name,":")] = '\0';

	/* look for an interface with this name */
	l = interface_list;
	while(l != NULL) {
		interface *f = (interface *)l->data;
		if(!strcmp(name, f->name)) {
			iface = f;
			break;
		}
		l = l->next;
	}

	if(iface == NULL) {
		if(name == "") {
			while(*pos != '\n' && *pos != '\0') pos++;
			if(*pos == '\0')
				return NULL;
			return ++pos;
		} else {
			iface = interface_new(name);
			interface_list = g_slist_append(interface_list, iface);
		}
	}

	/* skip over "name:" */
	while(*pos != ':' && *pos != '\0') pos++;

	if(*pos) pos++;
	/* get rx and tx counts */
		sscanf(pos, "%lu %*d %*d %*d %*d %*d %*d %*d %lu", &rx, &tx);

	/* skip to end of line */
	while(*pos != '\n' && *pos != '\0') pos++;
	if(*pos == '\0')
		return NULL;

	if(iface->tx_cur == 0)
		iface->tx_cur = tx;
	if(iface->rx_cur == 0)
		iface->rx_cur = rx;

	/* is the interface transmitting? */
	if(tx > iface->tx_cur)
		iface->txing = TRUE;
	else
		iface->txing = FALSE;

	/* is it receiving? */
	if(rx > iface->rx_cur)
		iface->rxing = TRUE;
	else
		iface->rxing = FALSE;

	/* set to newly read values */
	iface->tx_cur = tx;
	iface->rx_cur = rx;
	
	/* found the interface */
	if(rx>0 || tx>0)
		iface->found = TRUE;

	pos++;
	return pos;

}
