/* 
 * Blinky - a GTK1 network usage monitor
 *
 * Copyright (C) 2006, Lior Tubi <lior2b@gmail.com>
 *
 * This Blinky release is based on a work by Benjamin Miller <bmiller@users.sf.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <string.h>
#include <stdlib.h>
#include <gtk/gtk.h>
#include "blinky.h"

void create_graphics(void);

/****************************************************************************
 ****************************************************************************/

void get_setup(void) {
	int fd, length, cur = 0;
	static char buf[2048];
	char *pos, temp[32] = {'\0'}, filename[128] = {'\0'};
	interface *iface;

    /* default update interval is 140 milliseconds */
    options.interval = 140;
	options.options_window = NULL;

	/* create global image variables from pixmaps */
	create_graphics();

	/* omit lo interface */
	iface= interface_new("lo");
	iface->hidden = TRUE;
	interface_list = g_slist_append(interface_list, iface);

	/* get home folder */
	g_snprintf(filename, sizeof(filename), "%s/.blinky", getenv("HOME"));
	
	/* parse the config file */
	if((fd = open(filename, O_RDONLY)) != -1) {
		memset(buf, 0, sizeof(buf));

		if(read(fd, buf, sizeof(buf)) == -1) {
			perror("Error reading config file ~/.blinky");
		} else{
			pos = buf;
			
			while(*pos != '\0') {
				if(*pos != ' ' && *pos != '\n') {
					if(*pos == ',') {
						length = strlen(temp);
						if(length>0) {
							iface = interface_new(temp);
							iface->hidden = TRUE;
							interface_list = g_slist_append(interface_list, iface);
							memset(temp, '\0', sizeof(temp));
							cur = 0;
						}
					} else {
						temp[cur++] = *pos;
					}
				}
				++pos;
			}
			length = strlen(temp);
			if(length>0) {
				iface = interface_new(temp);
				iface->hidden = TRUE;
				interface_list = g_slist_append(interface_list, iface);
			}
		}
		close(fd);
	}

    return;
}

/****************************************************************************
 ****************************************************************************/

/* fills in the status of all interfaces in the interface list */
gboolean update_all(gpointer ptr) {
    GSList *l;

    /* first mark all interfaces as not found, then get_usage_all will mark
       them if it finds them */
    l = interface_list;
    while(l != NULL) {
		interface *f = l->data;
		f->found = FALSE;
		l = l->next;
    }

    /* get usage of all interfaces */
    get_usage_all();

    /* if any interfaces are left unmarked, deal with them */
    l = interface_list;
    while(l != NULL) {
		interface *f = l->data;
		if(f->found == FALSE) {
	    	f->tx_cur = f->rx_cur = 0;
	    	f->txing = f->rxing = FALSE;
		}
		l = l->next;
    }

    /* redraw all interfaces */
    l = interface_list;
    while(l != NULL) {
		interface *f = l->data;
		interface_draw(f->eventbox, NULL, f);
		l = l->next;
	}
    return TRUE;
}

/****************************************************************************
 ****************************************************************************/

void write_config_file(void) {
	GSList *l;
	int fd;
	char buf[2048] = {'\0'}, temp[16], home_folder[128];

	/* get home folder */
	g_snprintf(home_folder, sizeof(home_folder), "%s/.blinky", getenv("HOME"));

	l = interface_list;
	while(l != NULL) {
		if(strcmp((char *)(((interface*)l->data)->name), "lo")) {
			if((gboolean)(((interface *)l->data)->hidden)) {
				g_snprintf(temp, sizeof(temp), "%s,", (char *)(((interface *)l->data)->name));
				strncat(buf, temp, sizeof(temp));
			}
		}
		l = l->next;
	}
	buf[strlen(buf)] = '\n';

	/* write the config file */
	if((fd = creat(home_folder, S_IRUSR | S_IWRITE | S_IROTH | S_IRGRP)) != -1) {
		if(write(fd, buf, strlen(buf)*sizeof(char)) == -1)
			perror("Error writing config file ~/.blinky");
		close(fd);
	} else {
		perror("Error writing config file ~/.blinky");
	}
}

/****************************************************************************
 ****************************************************************************/

void create_graphics(void) {
	GtkStyle *style;

	/* computer graphics - bitmaps (.xpm) */
	char * idle_xpm[] = {
	"22 20 192 2","  	c None",". 	c #434343","+ 	c #737373","@ 	c #727272","# 	c #686868",
	"$ 	c #5A5A5A","% 	c #4A4A4A","& 	c #454545","* 	c #414141","= 	c #232323","- 	c #6E6E6E",
	"; 	c #848E96","> 	c #6E7C86",", 	c #74818C","' 	c #7B8791",") 	c #808D96","! 	c #87939B",
	"~ 	c #8D98A0","{ 	c #939EA5","] 	c #9AA3AB","^ 	c #9FA8AF","/ 	c #CFD0D0","( 	c #363636",
	"_ 	c #5A6B77",": 	c #263E50","< 	c #32495A","[ 	c #3F5464","} 	c #4B5F6E","| 	c #586A78",
	"1 	c #647682","2 	c #70808C","3 	c #7C8B96","4 	c #818E99","5 	c #C7C9CA","6 	c #3F3F3F",
	"7 	c #5C6C78","8 	c #253D4F","9 	c #304758","0 	c #3C5262","a 	c #495D6C","b 	c #4A5E6D",
	"c 	c #314859","d 	c #284051","e 	c #C0C2C3","f 	c #484848","g 	c #676767","h 	c #5E6E7A",
	"i 	c #304657","j 	c #364C5C","k 	c #294052","l 	c #C1C2C3","m 	c #505050","n 	c #555555",
	"o 	c #61717C","p 	c #C0C2C2","q 	c #595959","r 	c #677681","s 	c #455969","t 	c #415665",
	"u 	c #616161","v 	c #566774","w 	c #566875","x 	c #4A5A65","y 	c #3C4349","z 	c #6A6A6B",
	"A 	c #464646","B 	c #383838","C 	c #353535","D 	c #2D2D2D","E 	c #262626","F 	c #222222",
	"G 	c #191919","H 	c #474747","I 	c #DEDEDE","J 	c #E0E0E0","K 	c #6B6B6B","L 	c #B9BBBD",
	"M 	c #AAAEB1","N 	c #ABAFB2","O 	c #ACB0B3","P 	c #A9AEB0","Q 	c #A8ACAE","R 	c #A8ACAF",
	"S 	c #BABCBC","T 	c #393939","U 	c #2B2B2B","V 	c #545454","W 	c #585858","X 	c #5B5B5B",
	"Y 	c #636363","Z 	c #626262","` 	c #808D97"," .	c #25435A","..	c #324E64","+.	c #3F596E",
	"@.	c #4B6477","#.	c #586F80","$.	c #657A8A","%.	c #718493","&.	c #7D8F9D","*.	c #8999A6",
	"=.	c #B9C0C4","-.	c #4E4E4E",";.	c #0F0F0F",">.	c #A3A3A3",",.	c #ACACAC","'.	c #A4A4A4",
	").	c #828E98","!.	c #324F64","~.	c #3F5A6E","{.	c #627888","].	c #5D7384","^.	c #597081",
	"/.	c #536B7D","(.	c #A7AFB6","_.	c #2F2F2F",":.	c #6D6D6D","<.	c #868686","[.	c #707070",
	"}.	c #4B4B4B","|.	c #84909A","1.	c #304D63","2.	c #3C576C","3.	c #425C70","4.	c #365267",
	"5.	c #28455C","6.	c #9CA5AC","7.	c #5C5C5C","8.	c #6F6C40","9.	c #B0A87E","0.	c #5C4A13",
	"a.	c #494949","b.	c #849099","c.	c #244259","d.	c #304C62","e.	c #2F4B61","f.	c #A49D41",
	"g.	c #FDE94F","h.	c #BFA32D","i.	c #6D5E35","j.	c #85919A","k.	c #26445B","l.	c #6A6A6A",
	"m.	c #797536","n.	c #EFE558","o.	c #FDE449","p.	c #8B7E51","q.	c #444444","r.	c #87929B",
	"s.	c #415B6F","t.	c #455E72","u.	c #9EA7AE","v.	c #383813","w.	c #4F481C","x.	c #474015",
	"y.	c #2C2C2C","z.	c #B2B7BB","A.	c #939EA6","B.	c #96A0A8","C.	c #97A1A9","D.	c #97A1AA",
	"E.	c #98A2AB","F.	c #99A3AB","G.	c #9AA4AB","H.	c #9BA5AC","I.	c #BDC1C4","J.	c #282828",
	"K.	c #9E9E9E","L.	c #A7A7A7","M.	c #A9A9A9","N.	c #ABABAB","O.	c #ADADAD","P.	c #AFAFAF",
	"Q.	c #B0B0B0","R.	c #B1B1B1","S.	c #575757","T.	c #090909","U.	c #0A0A0A","V.	c #3C3C3C",
	"W.	c #4F4F4F","X.	c #515151","Y.	c #202020","Z.	c #060606","`.	c #171717"," +	c #B6B6B6",
	".+	c #B8B8B8","++	c #969696","@+	c #212121","#+	c #0D0D0D","$+	c #000000","%+	c #070707",
	"&+	c #3B3B3B",". + @ @ @ # $ % & & & * =                   ","- ; > , ' ) ! ~ { ] ^ / (                   ",
	"- _ : < [ } | 1 2 3 4 5 6                   ","- 7 8 9 0 a b [ c d 8 e f                   ","g h 8 i j k 8 8 8 8 8 l m                   ",
	"n o 8 k 8 8 8 8 8 8 8 p q                   ","* r s s s s s s s s t p u                   ","6 > v v v v w w w x y z A B B C D E F F F G ",
	"H I J J J J J J J K L M N O O P Q R P N S T ","U m V W W X Y Z # n `  ...+.@.#.$.%.&.*.=.-.","      ;.$ >.,.'.T -.). .!.~.@.#.{.].^./.(.n ",
	"      _.:.- <.:.[.}.|. .1.2.3.4.5. . . .6.7.","          8.9.0.  a.b.c.d.e. . . . . . .6.Y ","          f.g.h.i.* j.c.k. . . . . . . .6.l.",
	"          m.n.o.p.q.r.s.t.t.t.t.t.t.t.t.u.:.","            v.w.x.y.z.A.B.C.D.E.F.G.H.H.I.:.","                  J.K.L.M.N.O.P.Q.R.R.R.R.S.",
	"                    T.U.U.T.V.W.X.Y.U.U.U.Z.","                        `.#  +.+.+++D       ","                        @+#+$+$+$+$+%+&+    "};

char * tx_xpm[] = {
"22 20 200 2","  	c None",". 	c #434343","+ 	c #737373","@ 	c #727272","# 	c #686868",
"$ 	c #5A5A5A","% 	c #4A4A4A","& 	c #454545","* 	c #414141","= 	c #232323","- 	c #6E6E6E",
"; 	c #84919B","> 	c #6E7F8D",", 	c #748592","' 	c #7B8A97",") 	c #808F9C","! 	c #8796A1",
"~ 	c #8D9BA6","{ 	c #93A0AB","] 	c #9AA6B0","^ 	c #9FABB4","/ 	c #CFD0D0","( 	c #363636",
"_ 	c #5A6F7F",": 	c #26455C","< 	c #324F65","[ 	c #3F5A6F","} 	c #4B6578","| 	c #587081",
"1 	c #647A8B","2 	c #708594","3 	c #7C8F9D","4 	c #8193A0","5 	c #C7C9CA","6 	c #3F3F3F",
"7 	c #5C7180","8 	c #25445B","9 	c #304E64","0 	c #3C586D","a 	c #496376","b 	c #4A6477",
"c 	c #3F5A6E","d 	c #314F64","e 	c #28465D","f 	c #C0C2C3","g 	c #484848","h 	c #676767",
"i 	c #5E7281","j 	c #304D63","k 	c #365267","l 	c #29475E","m 	c #C1C2C3","n 	c #505050",
"o 	c #555555","p 	c #617584","q 	c #C0C2C2","r 	c #595959","s 	c #677A88","t 	c #455F73",
"u 	c #415C70","v 	c #616161","w 	c #6E808D","x 	c #566D7E","y 	c #566D7F","z 	c #4A5E6E",
"A 	c #3C454D","B 	c #6A6B6B","C 	c #464646","D 	c #383838","E 	c #353535","F 	c #2D2D2D",
"G 	c #262626","H 	c #222222","I 	c #191919","J 	c #474747","K 	c #DEDEDE","L 	c #E0E0E0",
"M 	c #6B6B6B","N 	c #BDC4B9","O 	c #B3C3AA","P 	c #B3C3AB","Q 	c #B5C5AC","R 	c #B5C4AC",
"S 	c #B2C1A9","T 	c #B0BFA8","U 	c #B1BFA8","V 	c #B2C0A9","W 	c #B4C1AB","X 	c #BDC2BA",
"Y 	c #393939","Z 	c #2B2B2B","` 	c #545454"," .	c #585858","..	c #5B5B5B","+.	c #636363",
"@.	c #626262","#.	c #9ACB80","$.	c #62D525","%.	c #6BD832","&.	c #75DA3F","*.	c #7DDD4B",
"=.	c #86DF58","-.	c #90E265",";.	c #98E471",">.	c #A2E67D",",.	c #AAE889","'.	c #C7DFB9",
").	c #4E4E4E","!.	c #0F0F0F","~.	c #A3A3A3","{.	c #ACACAC","].	c #A4A4A4","^.	c #9BCB82",
"/.	c #62D625","(.	c #6CD932","_.	c #75DB3F",":.	c #87E058","<.	c #8EE262","[.	c #8AE15D",
"}.	c #87E059","|.	c #84DF53","1.	c #B8D8A7","2.	c #2F2F2F","3.	c #6D6D6D","4.	c #868686",
"5.	c #707070","6.	c #4B4B4B","7.	c #9DCC84","8.	c #6AD830","9.	c #73DA3C","0.	c #76DB42",
"a.	c #6ED936","b.	c #64D628","c.	c #AFD39C","d.	c #5C5C5C","e.	c #6F6C40","f.	c #B0A87E",
"g.	c #5C4A13","h.	c #494949","i.	c #9CCA84","j.	c #61D524","k.	c #69D72F","l.	c #AFD29C",
"m.	c #A49D41","n.	c #FDE94F","o.	c #BFA32D","p.	c #6D5E35","q.	c #9DCA85","r.	c #63D626",
"s.	c #AED19C","t.	c #6A6A6A","u.	c #797536","v.	c #EFE558","w.	c #FDE449","x.	c #8B7E51",
"y.	c #444444","z.	c #9ECA87","A.	c #76DA41","B.	c #79DB45","C.	c #B1D29E","D.	c #383813",
"E.	c #4F481C","F.	c #474015","G.	c #2C2C2C","H.	c #BCD0B2","I.	c #A9D193","J.	c #ABD396",
"K.	c #ABD397","L.	c #ACD497","M.	c #ADD498","N.	c #AED599","O.	c #AED59A","P.	c #AFD59B",
"Q.	c #C5D3BD","R.	c #282828","S.	c #9E9E9E","T.	c #A7A7A7","U.	c #A9A9A9","V.	c #ABABAB",
"W.	c #ADADAD","X.	c #AFAFAF","Y.	c #B0B0B0","Z.	c #B1B1B1","`.	c #575757"," +	c #090909",
".+	c #0A0A0A","++	c #3C3C3C","@+	c #4F4F4F","#+	c #515151","$+	c #202020","%+	c #060606",
"&+	c #171717","*+	c #B6B6B6","=+	c #B8B8B8","-+	c #969696",";+	c #212121",">+	c #0D0D0D",
",+	c #000000","'+	c #070707",")+	c #3B3B3B",". + @ @ @ # $ % & & & * =                   ",
"- ; > , ' ) ! ~ { ] ^ / (                   ","- _ : < [ } | 1 2 3 4 5 6                   ","- 7 8 9 0 a b c d e 8 f g                   ",
"h i 8 j k l 8 8 8 8 8 m n                   ","o p 8 l 8 8 8 8 8 8 8 q r                   ","* s t t t t t t t t u q v                   ",
"6 w x x x x y y y z A B C D D E F G H H H I ","J K L L L L L L L M N O P Q R S T U V W X Y ","Z n `  . ...+.@.# o #.$.%.&.*.=.-.;.>.,.'.).",
"      !.$ ~.{.].Y ).^./.(._.*.:.<.[.}.|.1.o ","      2.3.- 4.3.5.6.7./.8.9.0.a.b./././.c.d.","          e.f.g.  h.i.j.8.k././././././.l.+.",
"          m.n.o.p.* q.j.r./././././././.s.t.","          u.v.w.x.y.z.A.B.B.B.B.B.B.B.B.C.3.","            D.E.F.G.H.I.J.K.L.M.N.O.P.P.Q.3.",
"                  R.S.T.U.V.W.X.Y.Z.Z.Z.Z.`.","                     +.+.+ +++@+#+$+.+.+.+%+","                        &+# *+=+=+-+F       ",
"                        ;+>+,+,+,+,+'+)+    "};

char * rx_xpm[] = {
"22 20 191 2","  	c None",". 	c #434343","+ 	c #737373","@ 	c #727272","# 	c #686868",
"$ 	c #5A5A5A","% 	c #4A4A4A","& 	c #454545","* 	c #414141","= 	c #232323","- 	c #6E6E6E",
"; 	c #9ED084","> 	c #91D46E",", 	c #95D674","' 	c #9AD87B",") 	c #9FD980","! 	c #A4DB87",
"~ 	c #A9DD8D","{ 	c #ADDF93","] 	c #B2E19A","^ 	c #B6E39F","/ 	c #D0D3CF","( 	c #363636",
"_ 	c #84D35A",": 	c #63D726","< 	c #6CD932","[ 	c #75DB3F","} 	c #7DDD4B","| 	c #87E058",
"1 	c #90E264","2 	c #98E470","3 	c #A1E77C","4 	c #A4E781","5 	c #CACFC7","6 	c #3F3F3F",
"7 	c #85D35C","8 	c #62D625","9 	c #6AD830","0 	c #73DA3C","a 	c #7CDC49","b 	c #7DDD4A",
"c 	c #75DA3F","d 	c #6BD831","e 	c #64D628","f 	c #C3C9C0","g 	c #484848","h 	c #676767",
"i 	c #86D35E","j 	c #6ED936","k 	c #65D629","l 	c #C4C9C1","m 	c #505050","n 	c #555555",
"o 	c #89D461","p 	c #595959","q 	c #8DD567","r 	c #79DB45","s 	c #76DA41","t 	c #616161",
"u 	c #91D26E","v 	c #83D956","w 	c #71BC4A","x 	c #4E713C","y 	c #6B6D6A","z 	c #464646",
"A 	c #383838","B 	c #353535","C 	c #2D2D2D","D 	c #262626","E 	c #222222","F 	c #191919",
"G 	c #474747","H 	c #DEDEDE","I 	c #E0E0E0","J 	c #6B6B6B","K 	c #B9BBBD","L 	c #AAAEB1",
"M 	c #ABAFB2","N 	c #ACB0B3","O 	c #A9AEB0","P 	c #A8ACAE","Q 	c #A8ACAF","R 	c #BABCBC",
"S 	c #393939","T 	c #2B2B2B","U 	c #545454","V 	c #585858","W 	c #5B5B5B","X 	c #636363",
"Y 	c #626262","Z 	c #808D97","` 	c #25435A"," .	c #324E64","..	c #3F596E","+.	c #4B6477",
"@.	c #586F80","#.	c #657A8A","$.	c #718493","%.	c #7D8F9D","&.	c #8999A6","*.	c #B9C0C4",
"=.	c #4E4E4E","-.	c #0F0F0F",";.	c #A3A3A3",">.	c #ACACAC",",.	c #A4A4A4","'.	c #828E98",
").	c #324F64","!.	c #3F5A6E","~.	c #627888","{.	c #5D7384","].	c #597081","^.	c #536B7D",
"/.	c #A7AFB6","(.	c #2F2F2F","_.	c #6D6D6D",":.	c #868686","<.	c #707070","[.	c #4B4B4B",
"}.	c #84909A","|.	c #304D63","1.	c #3C576C","2.	c #425C70","3.	c #365267","4.	c #28455C",
"5.	c #9CA5AC","6.	c #5C5C5C","7.	c #6F6C40","8.	c #B0A87E","9.	c #5C4A13","0.	c #494949",
"a.	c #849099","b.	c #244259","c.	c #304C62","d.	c #2F4B61","e.	c #A49D41","f.	c #FDE94F",
"g.	c #BFA32D","h.	c #6D5E35","i.	c #85919A","j.	c #26445B","k.	c #6A6A6A","l.	c #797536",
"m.	c #EFE558","n.	c #FDE449","o.	c #8B7E51","p.	c #444444","q.	c #87929B","r.	c #415B6F",
"s.	c #455E72","t.	c #9EA7AE","u.	c #383813","v.	c #4F481C","w.	c #474015","x.	c #2C2C2C",
"y.	c #B2B7BB","z.	c #939EA6","A.	c #96A0A8","B.	c #97A1A9","C.	c #97A1AA","D.	c #98A2AB",
"E.	c #99A3AB","F.	c #9AA4AB","G.	c #9BA5AC","H.	c #BDC1C4","I.	c #282828","J.	c #9E9E9E",
"K.	c #A7A7A7","L.	c #A9A9A9","M.	c #ABABAB","N.	c #ADADAD","O.	c #AFAFAF","P.	c #B0B0B0",
"Q.	c #B1B1B1","R.	c #575757","S.	c #090909","T.	c #0A0A0A","U.	c #3C3C3C","V.	c #4F4F4F",
"W.	c #515151","X.	c #202020","Y.	c #060606","Z.	c #171717","`.	c #B6B6B6"," +	c #B8B8B8",
".+	c #969696","++	c #212121","@+	c #0D0D0D","#+	c #000000","$+	c #070707","%+	c #3B3B3B",
". + @ @ @ # $ % & & & * =                   ","- ; > , ' ) ! ~ { ] ^ / (                   ","- _ : < [ } | 1 2 3 4 5 6                   ",
"- 7 8 9 0 a b c d e 8 f g                   ","h i 8 9 j k 8 8 8 8 8 l m                   ","n o 8 k 8 8 8 8 8 8 8 f p                   ",
"* q r r r r r r r r s f t                   ","6 u v v v v v v v w x y z A A B C D E E E F ","G H I I I I I I I J K L M N N O P Q O M R S ",
"T m U V V W X Y # n Z `  ...+.@.#.$.%.&.*.=.","      -.$ ;.>.,.S =.'.` ).!.+.@.~.{.].^./.n ","      (._.- :._.<.[.}.` |.1.2.3.4.` ` ` 5.6.",
"          7.8.9.  0.a.b.c.d.` ` ` ` ` ` 5.X ","          e.f.g.h.* i.b.j.` ` ` ` ` ` ` 5.k.","          l.m.n.o.p.q.r.s.s.s.s.s.s.s.s.t._.",
"            u.v.w.x.y.z.A.B.C.D.E.F.G.G.H._.","                  I.J.K.L.M.N.O.P.Q.Q.Q.Q.R.","                    S.T.T.S.U.V.W.X.T.T.T.Y.",
"                        Z.# `. + +.+C       ","                        ++@+#+#+#+#+$+%+    "};

char * txrx_xpm[] = {
"22 20 185 2","  	c None",". 	c #434343","+ 	c #737373","@ 	c #727272","# 	c #686868",
"$ 	c #5A5A5A","% 	c #4A4A4A","& 	c #454545","* 	c #414141","= 	c #232323","- 	c #6E6E6E",
"; 	c #9ED084","> 	c #91D46E",", 	c #95D674","' 	c #9AD87B",") 	c #9FD980","! 	c #A4DB87",
"~ 	c #A9DD8D","{ 	c #ADDF93","] 	c #B2E19A","^ 	c #B6E39F","/ 	c #D0D3CF","( 	c #363636",
"_ 	c #84D35A",": 	c #63D726","< 	c #6CD932","[ 	c #75DB3F","} 	c #7DDD4B","| 	c #87E058",
"1 	c #90E264","2 	c #98E470","3 	c #A1E77C","4 	c #A4E781","5 	c #CACFC7","6 	c #3F3F3F",
"7 	c #85D35C","8 	c #62D625","9 	c #6AD830","0 	c #73DA3C","a 	c #7CDC49","b 	c #7DDD4A",
"c 	c #75DA3F","d 	c #6BD831","e 	c #64D628","f 	c #C3C9C0","g 	c #484848","h 	c #676767",
"i 	c #86D35E","j 	c #6ED936","k 	c #65D629","l 	c #C4C9C1","m 	c #505050","n 	c #555555",
"o 	c #89D461","p 	c #595959","q 	c #8DD567","r 	c #79DB45","s 	c #76DA41","t 	c #616161",
"u 	c #91D26E","v 	c #83D956","w 	c #71BC4A","x 	c #4E713C","y 	c #6B6D6A","z 	c #464646",
"A 	c #383838","B 	c #353535","C 	c #2D2D2D","D 	c #262626","E 	c #222222","F 	c #191919",
"G 	c #474747","H 	c #DEDEDE","I 	c #E0E0E0","J 	c #6B6B6B","K 	c #BDC4B9","L 	c #B3C3AA",
"M 	c #B3C3AB","N 	c #B5C5AC","O 	c #B5C4AC","P 	c #B2C1A9","Q 	c #B0BFA8","R 	c #B1BFA8",
"S 	c #B2C0A9","T 	c #B4C1AB","U 	c #BDC2BA","V 	c #393939","W 	c #2B2B2B","X 	c #545454",
"Y 	c #585858","Z 	c #5B5B5B","` 	c #636363"," .	c #626262","..	c #9ACB80","+.	c #62D525",
"@.	c #6BD832","#.	c #86DF58","$.	c #90E265","%.	c #98E471","&.	c #A2E67D","*.	c #AAE889",
"=.	c #C7DFB9","-.	c #4E4E4E",";.	c #0F0F0F",">.	c #A3A3A3",",.	c #ACACAC","'.	c #A4A4A4",
").	c #9BCB82","!.	c #8EE262","~.	c #8AE15D","{.	c #87E059","].	c #84DF53","^.	c #B8D8A7",
"/.	c #2F2F2F","(.	c #6D6D6D","_.	c #868686",":.	c #707070","<.	c #4B4B4B","[.	c #9DCC84",
"}.	c #76DB42","|.	c #AFD39C","1.	c #5C5C5C","2.	c #6F6C40","3.	c #B0A87E","4.	c #5C4A13",
"5.	c #494949","6.	c #9CCA84","7.	c #61D524","8.	c #69D72F","9.	c #AFD29C","0.	c #A49D41",
"a.	c #FDE94F","b.	c #BFA32D","c.	c #6D5E35","d.	c #9DCA85","e.	c #63D626","f.	c #AED19C",
"g.	c #6A6A6A","h.	c #797536","i.	c #EFE558","j.	c #FDE449","k.	c #8B7E51","l.	c #444444",
"m.	c #9ECA87","n.	c #B1D29E","o.	c #383813","p.	c #4F481C","q.	c #474015","r.	c #2C2C2C",
"s.	c #BCD0B2","t.	c #A9D193","u.	c #ABD396","v.	c #ABD397","w.	c #ACD497","x.	c #ADD498",
"y.	c #AED599","z.	c #AED59A","A.	c #AFD59B","B.	c #C5D3BD","C.	c #282828","D.	c #9E9E9E",
"E.	c #A7A7A7","F.	c #A9A9A9","G.	c #ABABAB","H.	c #ADADAD","I.	c #AFAFAF","J.	c #B0B0B0",
"K.	c #B1B1B1","L.	c #575757","M.	c #090909","N.	c #0A0A0A","O.	c #3C3C3C","P.	c #4F4F4F",
"Q.	c #515151","R.	c #202020","S.	c #060606","T.	c #171717","U.	c #B6B6B6","V.	c #B8B8B8",
"W.	c #969696","X.	c #212121","Y.	c #0D0D0D","Z.	c #000000","`.	c #070707"," +	c #3B3B3B",
". + @ @ @ # $ % & & & * =                   ","- ; > , ' ) ! ~ { ] ^ / (                   ","- _ : < [ } | 1 2 3 4 5 6                   ",
"- 7 8 9 0 a b c d e 8 f g                   ","h i 8 9 j k 8 8 8 8 8 l m                   ","n o 8 k 8 8 8 8 8 8 8 f p                   ",
"* q r r r r r r r r s f t                   ","6 u v v v v v v v w x y z A A B C D E E E F ","G H I I I I I I I J K L M N O P Q R S T U V ",
"W m X Y Y Z `  .# n ..+.@.c } #.$.%.&.*.=.-.","      ;.$ >.,.'.V -.).8 < [ } | !.~.{.].^.n ","      /.(.- _.(.:.<.[.8 9 0 }.j e 8 8 8 |.1.",
"          2.3.4.  5.6.7.9 8.8 8 8 8 8 8 9.` ","          0.a.b.c.* d.7.e.8 8 8 8 8 8 8 f.g.","          h.i.j.k.l.m.s r r r r r r r r n.(.",
"            o.p.q.r.s.t.u.v.w.x.y.z.A.A.B.(.","                  C.D.E.F.G.H.I.J.K.K.K.K.L.","                    M.N.N.M.O.P.Q.R.N.N.N.S.",
"                        T.# U.V.V.W.C       ","                        X.Y.Z.Z.Z.Z.`. +    "};

	style = gtk_widget_get_style(mainbox);
	img_txrx.pixmap = gdk_pixmap_create_from_xpm_d(gtk_widget_get_parent_window(mainbox), &img_txrx.bitmap, &style->bg[GTK_STATE_NORMAL], txrx_xpm);
	img_rx.pixmap = gdk_pixmap_create_from_xpm_d(gtk_widget_get_parent_window(mainbox), &img_rx.bitmap, &style->bg[GTK_STATE_NORMAL], rx_xpm);
	img_tx.pixmap = gdk_pixmap_create_from_xpm_d(gtk_widget_get_parent_window(mainbox), &img_tx.bitmap, &style->bg[GTK_STATE_NORMAL], tx_xpm);
	img_idle.pixmap = gdk_pixmap_create_from_xpm_d(gtk_widget_get_parent_window(mainbox), &img_idle.bitmap, &style->bg[GTK_STATE_NORMAL], idle_xpm);

}

/****************************************************************************
 ****************************************************************************/

/* This function takes a unsigned long long number of bytes, and writes
 * a nicly formatted version of the size in Kb, MB, GB, etc... into a provided
 * buffer.  To insure no buffer overflows occur one must also pass in the size
 * of the buffer provided, which at maximum should only need to be 11 including
 * the terminating NULL */
void format_bytes(unsigned long long bytes, char *buffer, int size) {
	if(bytes > 0.0) { // bytes range 
		if(bytes > 1024.0) { // KB range 
			if(bytes > 1048576.0) { // MB range 
				if(bytes > 1073741824.0) { // GB range
					if(bytes > 1099511627776.0) { // TB range
						if(bytes > 1125899906842624.0) { // PB range
							if(bytes > 1152921504606846976.0) { // EB range
								if(bytes > 1180591620717411303424.0) { // ZB range
									if(bytes > 12089258196146291747066176.0) { // YB range
										g_snprintf(buffer, size, 
										"%.2f YB",
										(float)bytes/12089258196146291747066176.0);
									} else  // ZB range
										g_snprintf(buffer, size, 
										"%.2f ZB",
										(float)bytes/1180591620717411303424.0);
								} else  // EB range
									g_snprintf(buffer, size, 
									"%.2f EB",
									(float)bytes/1152921504606846976.0);
							} else // PB range
								g_snprintf(buffer, size, 
								"%.2f PB",
								(float)bytes/1125899906842624.0);
						} else // TB range
							g_snprintf(buffer, size, 
							"%.2f TB",
							(float)bytes/1099511627776.0);
					} else // GB range
						g_snprintf(buffer, size, 
						"%.2f GB",
						(float)bytes/1073741824.0);
				} else //MB range
					g_snprintf(buffer, size, 
					"%.2f MB",
					(float)bytes/1048576.0);
			} else //KB range
				g_snprintf(buffer, size, 
				"%.2f KB",
				(float)bytes/1024.0);
		} else // byte range
			g_snprintf(buffer, size, 
			"%llu bytes",
			bytes);
	} else // none
		g_snprintf(buffer, size, 
		"None");
	return;
}
